@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2011-2016 Phusion Holding B.V.
+ *  Copyright (c) 2011-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -290,17 +290,18 @@ void
 Controller::writeOtherExceptionErrorResponse(Client *client, Request *req, const ExceptionPtr &e) {
 	TRACE_POINT();
 	string typeName;
+	const oxt::tracable_exception &eptr = *e;
 	#ifdef CXX_ABI_API_AVAILABLE
 		int status;
-		char *tmp = abi::__cxa_demangle(typeid(*e).name(), 0, 0, &status);
+		char *tmp = abi::__cxa_demangle(typeid(eptr).name(), 0, 0, &status);
 		if (tmp != NULL) {
 			typeName = tmp;
 			free(tmp);
 		} else {
-			typeName = typeid(*e).name();
+			typeName = typeid(eptr).name();
 		}
 	#else
-		typeName = typeid(*e).name();
+		typeName = typeid(eptr).name();
 	#endif
 
 	const unsigned int exceptionMessageLen = strlen(e->what());
@@ -362,7 +363,7 @@ Controller::endRequestWithErrorResponse(Client **c, Request **r, const StaticStr
 		}
 	} else {
 		try {
-			data = renderer.renderWithoutDetails();
+			data = renderer.renderWithoutDetails(e);
 		} catch (const SystemException &e2) {
 			SKC_ERROR(client, "Cannot render an error page: " << e2.what() <<
 				"\n" << e2.backtrace());
@@ -376,7 +377,7 @@ Controller::endRequestWithErrorResponse(Client **c, Request **r, const StaticStr
 bool
 Controller::friendlyErrorPagesEnabled(Request *req) {
 	bool defaultValue;
-	string defaultStr = agentsOptions->get("friendly_error_pages");
+	const StaticString &defaultStr = req->configCache->friendlyErrorPages;
 	if (defaultStr == "auto") {
 		defaultValue = (req->options.environment == "development");
 	} else {
