@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2015 Phusion Holding B.V.
+ *  Copyright (c) 2010-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -42,6 +42,9 @@
 	#define HAVE_FLOCK
 #endif
 
+#ifdef __linux__
+#include <sys/prctl.h>
+#endif
 #include <sys/select.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -122,6 +125,7 @@ namespace WatchdogAgent {
 		int apiServerFds[SERVER_KIT_MAX_SERVER_ENDPOINTS];
 		BackgroundEventLoop *bgloop;
 		ServerKit::Context *serverKitContext;
+		ServerKit::HttpServerSchema apiServerSchema;
 		ApiServer *apiServer;
 
 		WorkingObjects()
@@ -148,9 +152,7 @@ using namespace Passenger::WatchdogAgent;
 
 static VariantMap *agentsOptions;
 static WorkingObjects *workingObjects;
-static string oldOomScore;
 
-static void setOomScore(const StaticString &score);
 static void cleanup(const WorkingObjectsPtr &wo);
 
 #include "AgentWatcher.cpp"
@@ -162,7 +164,7 @@ static void cleanup(const WorkingObjectsPtr &wo);
 /***** Functions *****/
 
 static FILE *
-openOomAdjFile(const char *mode, OomFileType &type) {
+openOomAdjFileGetType(const char *mode, OomFileType &type) {
 	FILE *f = fopen("/proc/self/oom_score_adj", mode);
 	if (f == NULL) {
 		f = fopen("/proc/self/oom_adj", mode);
@@ -178,27 +180,13 @@ openOomAdjFile(const char *mode, OomFileType &type) {
 	}
 }
 
-/**
- * Linux-only way to change OOM killer configuration for
- * current process. Requires root privileges, which we
- * should have.
- */
-static void
-setOomScore(const StaticString &score) {
-	if (score.empty()) {
-		return;
-	}
-
-	FILE *f;
-	OomFileType type;
-
-	f = openOomAdjFile("w", type);
-	if (f != NULL) {
-		size_t ret = fwrite(score.data(), 1, score.size(), f);
-		// We can't do anything about failures, so ignore compiler
-		// warnings about not doing anything with the result.
-		(void) ret;
-		fclose(f);
+static FILE *
+openOomAdjFileForcedType(const char *mode, OomFileType &type) {
+	if (type == OOM_SCORE_ADJ) {
+		return fopen("/proc/self/oom_score_adj", mode);
+	} else {
+		assert(type == OOM_ADJ);
+		return fopen("/proc/self/oom_adj", mode);
 	}
 }
 
@@ -211,10 +199,14 @@ setOomScoreNeverKill() {
 	FILE *f;
 	OomFileType type;
 
-	f = openOomAdjFile("r", type);
+	f = openOomAdjFileGetType("r", type);
 	if (f == NULL) {
 		return "";
 	}
+	// mark if this is a legacy score so we won't try to write it as OOM_SCORE_ADJ
+	if (type == OOM_ADJ) {
+		oldScore.append("l");
+	}
 	char buf[1024];
 	size_t bytesRead;
 	while (true) {
@@ -230,7 +222,7 @@ setOomScoreNeverKill() {
 	}
 	fclose(f);
 
-	f = openOomAdjFile("w", type);
+	f = openOomAdjFileForcedType("w", type);
 	if (f == NULL) {
 		return "";
 	}
@@ -334,12 +326,18 @@ waitForStarterProcessOrWatchers(const WorkingObjectsPtr &wo, vector<AgentWatcher
 	}
 }
 
+string
+relative(string filename){
+	string dir = filename.substr(filename.find_last_of('/')+1);
+	return dir;
+}
+
 static vector<pid_t>
 readCleanupPids(const WorkingObjectsPtr &wo) {
 	vector<pid_t> result;
 
 	foreach (string filename, wo->cleanupPidfiles) {
-		FILE *f = fopen(filename.c_str(), "r");
+		FILE *f = fopen(relative(filename).c_str(), "r");
 		if (f != NULL) {
 			char buf[33];
 			size_t ret;
@@ -364,7 +362,10 @@ static void
 killCleanupPids(const vector<pid_t> &cleanupPids) {
 	foreach (pid_t pid, cleanupPids) {
 		P_DEBUG("Sending SIGTERM to cleanup PID " << pid);
-		kill(pid, SIGTERM);
+		if(kill(pid, SIGTERM) == -1){
+			int e = errno;
+			P_WARN("Failed to send SIGTERM to " << pid << ", error: " << e << " " << strerror(e));
+		}
 	}
 }
 
@@ -387,8 +388,8 @@ deletePidFile(const WorkingObjectsPtr &wo) {
 
 static void
 cleanupAgentsInBackground(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &watchers, char *argv[]) {
-	this_thread::disable_interruption di;
-	this_thread::disable_syscall_interruption dsi;
+	boost::this_thread::disable_interruption di;
+	boost::this_thread::disable_syscall_interruption dsi;
 	pid_t pid;
 	int e;
 
@@ -397,7 +398,7 @@ cleanupAgentsInBackground(const WorkingObjectsPtr &wo, vector<AgentWatcherPtr> &
 		// Child
 		try {
 			vector<AgentWatcherPtr>::const_iterator it;
-			Timer timer(false);
+			Timer<SystemTime::GRAN_10MSEC> timer(false);
 			fd_set fds, fds2;
 			int max, agentProcessesDone;
 			unsigned long long deadline = 30000; // miliseconds
@@ -776,11 +777,12 @@ initializeBareEssentials(int argc, char *argv[], WorkingObjectsPtr &wo) {
 	 * for this watchdog. Note that the OOM score is inherited by child processes
 	 * so we need to restore it after each fork().
 	 */
-	oldOomScore = setOomScoreNeverKill();
+	string oldOomScore = setOomScoreNeverKill();
 
 	agentsOptions = new VariantMap();
 	*agentsOptions = initializeAgent(argc, &argv, SHORT_PROGRAM_NAME " watchdog",
 		parseOptions, NULL, 2);
+	agentsOptions->set("original_oom_score", oldOomScore);
 
 	// Start all sub-agents with this environment variable.
 	setenv("PASSENGER_USE_FEEDBACK_FD", "true", 1);
@@ -900,6 +902,18 @@ openReportFile(const WorkingObjectsPtr &wo) {
 	}
 }
 
+static void
+chdirToTmpDir() {
+	vector<string> pidfiles = agentsOptions->getStrSet("cleanup_pidfiles", false);
+	if (pidfiles.size() > 0) {
+		string str = pidfiles.front();
+		string dir = str.substr(0,str.find_last_of('/'));
+		if (dir != "" && chdir(dir.c_str()) == -1) {
+			throw RuntimeException("Cannot change working directory to " + dir);
+		}
+	}
+}
+
 static void
 lowerPrivilege() {
 	TRACE_POINT();
@@ -935,7 +949,9 @@ lowerPrivilege() {
 				"to that of user '" + userName + "' and group '" + groupName +
 				"': cannot set user ID to " + toString(pwUser->pw_uid), e);
 		}
-
+#ifdef __linux__
+		prctl(PR_SET_DUMPABLE, 1);
+#endif
 		setenv("USER", pwUser->pw_name, 1);
 		setenv("HOME", pwUser->pw_dir, 1);
 		setenv("UID", toString(gid).c_str(), 1);
@@ -998,16 +1014,6 @@ initializeWorkingObjects(const WorkingObjectsPtr &wo, InstanceDirToucherPtr &ins
 	if (options.get("integration_mode") == "standalone") {
 		instanceOptions.properties["standalone_engine"] = options.get("standalone_engine");
 	}
-	if (options.has("web_server_config_files")) {
-		vector<string> configFiles = options.getStrSet("web_server_config_files");
-		Json::Value array(Json::arrayValue);
-
-		foreach (string configFile, configFiles) {
-			array.append(configFile);
-		}
-
-		instanceOptions.properties["web_server_config_files"] = array;
-	}
 	wo->instanceDir = boost::make_shared<InstanceDirectory>(instanceOptions,
 		options.get("instance_registry_dir"));
 	options.set("instance_dir", wo->instanceDir->getPath());
@@ -1150,10 +1156,11 @@ initializeApiServer(const WorkingObjectsPtr &wo) {
 		absolutizePath(options.get("data_buffer_dir"));
 
 	UPDATE_TRACE_POINT();
-	wo->apiServer = new ApiServer(wo->serverKitContext);
+	wo->apiServer = new ApiServer(wo->serverKitContext, wo->apiServerSchema);
 	wo->apiServer->apiAccountDatabase = &wo->apiAccountDatabase;
 	wo->apiServer->exitEvent = &wo->exitEvent;
 	wo->apiServer->fdPassingPassword = options.get("watchdog_fd_passing_password");
+	wo->apiServer->initialize();
 	for (unsigned int i = 0; i < apiAddresses.size(); i++) {
 		wo->apiServer->listen(wo->apiServerFds[i]);
 	}
@@ -1285,6 +1292,7 @@ watchdogMain(int argc, char *argv[]) {
 		maybeDaemonize();
 		createPidFile();
 		openReportFile(wo);
+		chdirToTmpDir();
 		lowerPrivilege();
 		initializeWorkingObjects(wo, instanceDirToucher, uidBeforeLoweringPrivilege);
 		initializeAgentWatchers(wo, watchers);
@@ -1324,8 +1332,8 @@ watchdogMain(int argc, char *argv[]) {
 		runHookScriptAndThrowOnError("after_watchdog_initialization");
 
 		UPDATE_TRACE_POINT();
-		this_thread::disable_interruption di;
-		this_thread::disable_syscall_interruption dsi;
+		boost::this_thread::disable_interruption di;
+		boost::this_thread::disable_syscall_interruption dsi;
 		bool shouldExitGracefully = waitForStarterProcessOrWatchers(wo, watchers);
 		if (shouldExitGracefully) {
 			/* Fork a child process which cleans up all the agent processes in
