@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2015 Phusion Holding B.V.
+ *  Copyright (c) 2010-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -79,7 +79,7 @@ private:
 
 		~SharedData() {
 			if (fd >= 0 && autoClose) {
-				this_thread::disable_syscall_interruption dsi;
+				boost::this_thread::disable_syscall_interruption dsi;
 				syscalls::close(fd);
 				P_LOG_FILE_DESCRIPTOR_CLOSE(fd);
 			}
@@ -87,7 +87,7 @@ private:
 
 		void close(bool checkErrors = true) {
 			if (fd >= 0) {
-				this_thread::disable_syscall_interruption dsi;
+				boost::this_thread::disable_syscall_interruption dsi;
 				int theFd = fd;
 				fd = -1;
 				safelyClose(theFd, !checkErrors);
@@ -292,7 +292,7 @@ public:
 	}
 
 	~EventFd() {
-		this_thread::disable_syscall_interruption dsi;
+		boost::this_thread::disable_syscall_interruption dsi;
 		syscalls::close(reader);
 		syscalls::close(writer);
 		P_LOG_FILE_DESCRIPTOR_CLOSE(reader);
