@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2014-2016 Phusion Holding B.V.
+ *  Copyright (c) 2014-2017 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -60,6 +60,30 @@ extern const char DEFAULT_INTERNAL_SERVER_ERROR_RESPONSE[];
 extern const unsigned int DEFAULT_INTERNAL_SERVER_ERROR_RESPONSE_SIZE;
 
 
+class HttpServerSchema: public BaseServerSchema {
+private:
+	void initialize() {
+		using namespace ConfigKit;
+
+		add("request_freelist_limit", UINT_TYPE, OPTIONAL, 1024);
+	}
+
+public:
+	HttpServerSchema()
+		: BaseServerSchema(true)
+	{
+		initialize();
+		finalize();
+	}
+
+	HttpServerSchema(bool _subclassing)
+		: BaseServerSchema(true)
+	{
+		initialize();
+	}
+};
+
+
 template< typename DerivedServer, typename Client = HttpClient<HttpRequest> >
 class HttpServer: public BaseServer<DerivedServer, Client> {
 public:
@@ -544,15 +568,6 @@ private:
 
 	/***** Miscellaneous *****/
 
-	char *appendLStringData(char *pos, const char *end, const LString *lstr) {
-		const LString::Part *part = lstr->start;
-		while (part != NULL) {
-			pos = appendData(pos, end, part->data, part->size);
-			part = part->next;
-		}
-		return pos;
-	}
-
 	void writeDefault500Response(Client *client, Request *req) {
 		writeSimpleResponse(client, 500, NULL, DEFAULT_INTERNAL_SERVER_ERROR_RESPONSE);
 	}
@@ -926,6 +941,13 @@ protected:
 		req->bodyChannel.deinitialize();
 	}
 
+	virtual void onConfigChange(const ConfigKit::Store *oldConfig) {
+		ParentClass::onConfigChange(oldConfig);
+		const ConfigKit::Store &config = this->config;
+
+		requestFreelistLimit = config["request_freelist_limit"].asUInt();
+	}
+
 
 	/***** Misc *****/
 
@@ -935,8 +957,9 @@ protected:
 	}
 
 public:
-	HttpServer(Context *context)
-		: ParentClass(context),
+	HttpServer(Context *context, const HttpServerSchema &schema,
+		const Json::Value &initialConfig = Json::Value())
+		: ParentClass(context, schema, initialConfig),
 		  freeRequestCount(0),
 		  requestFreelistLimit(1024),
 		  totalRequestsBegun(0),
@@ -975,7 +998,7 @@ public:
 
 	/***** Request manipulation *****/
 
-		/** Increase request reference count. */
+	/** Increase request reference count. */
 	void refRequest(Request *req, const char *file, unsigned int line) {
 		int oldRefcount = req->refcount.fetch_add(1, boost::memory_order_relaxed);
 		SKC_TRACE_WITH_POS(static_cast<Client *>(req->client), 3, file, line,
@@ -1062,7 +1085,7 @@ public:
 			pos = appendData(pos, end, P_STATIC_STRING("Content-Type: text/html; charset=UTF-8\r\n"));
 		} else {
 			pos = appendData(pos, end, P_STATIC_STRING("Content-Type: "));
-			pos = appendLStringData(pos, end, value);
+			pos = appendData(pos, end, value);
 			pos = appendData(pos, end, P_STATIC_STRING("\r\n"));
 		}
 
@@ -1074,7 +1097,7 @@ public:
 			gmtime_r(&the_time, &the_tm);
 			pos += strftime(pos, end - pos, "%a, %d %b %Y %H:%M:%S %z", &the_tm);
 		} else {
-			pos = appendLStringData(pos, end, value);
+			pos = appendData(pos, end, value);
 		}
 		pos = appendData(pos, end, P_STATIC_STRING("\r\n"));
 
@@ -1087,7 +1110,7 @@ public:
 			}
 		} else {
 			pos = appendData(pos, end, P_STATIC_STRING("Connection: "));
-			pos = appendLStringData(pos, end, value);
+			pos = appendData(pos, end, value);
 			pos = appendData(pos, end, P_STATIC_STRING("\r\n"));
 			if (!psg_lstr_cmp(value, P_STATIC_STRING("Keep-Alive"))
 			 && !psg_lstr_cmp(value, P_STATIC_STRING("keep-alive")))
@@ -1101,7 +1124,7 @@ public:
 		if (value == NULL) {
 			pos += snprintf(pos, end - pos, "%u", (unsigned int) body.size());
 		} else {
-			pos = appendLStringData(pos, end, value);
+			pos = appendData(pos, end, value);
 		}
 		pos = appendData(pos, end, P_STATIC_STRING("\r\n"));
 
@@ -1113,9 +1136,9 @@ public:
 				 && !psg_lstr_cmp(&it->header->key, P_STATIC_STRING("connection"))
 				 && !psg_lstr_cmp(&it->header->key, P_STATIC_STRING("content-length")))
 				{
-					pos = appendLStringData(pos, end, &it->header->origKey);
+					pos = appendData(pos, end, &it->header->origKey);
 					pos = appendData(pos, end, P_STATIC_STRING(": "));
-					pos = appendLStringData(pos, end, &it->header->val);
+					pos = appendData(pos, end, &it->header->val);
 					pos = appendData(pos, end, P_STATIC_STRING("\r\n"));
 				}
 				it.next();
@@ -1187,19 +1210,6 @@ public:
 
 	/***** Configuration and introspection *****/
 
-	virtual void configure(const Json::Value &doc) {
-		ParentClass::configure(doc);
-		if (doc.isMember("request_freelist_limit")) {
-			requestFreelistLimit = doc["request_freelist_limit"].asUInt();
-		}
-	}
-
-	virtual Json::Value getConfigAsJson() const {
-		Json::Value doc = ParentClass::getConfigAsJson();
-		doc["request_freelist_limit"] = requestFreelistLimit;
-		return doc;
-	}
-
 	virtual Json::Value inspectStateAsJson() const {
 		Json::Value doc = ParentClass::inspectStateAsJson();
 		doc["free_request_count"] = freeRequestCount;
